@@ -115,13 +115,15 @@ class StompAdapterImpl(object):
             )
 
             # Make sure the heart-beat interval is sane
+            if cx != 0:
+                cx = max(cx, 1000)
             if cy != 0:
                 cy = max(cy, 1000)
 
-            # The server can send a heart-beat every cy ms and doesn't want
-            # to receive any heart-beat from the client.
-            resp.headers[stomp.Headers.HEARTBEAT] = "%d,0" % (cy,)
-            dispatcher.setHeartBeat(cy)
+            # The server can send a heartbeat every cy ms and get a heartbeat
+            # every cx ms.
+            resp.headers[stomp.Headers.HEARTBEAT] = "%d,%d" % (cy, cx)
+            dispatcher.setHeartBeat(cy, cx)
 
         self.queue_frame(resp)
         self._reactor.wakeup()
@@ -241,6 +243,10 @@ class StompAdapterImpl(object):
             pass
         dispatcher.connection.handleMessage(request, flow_id)
 
+    def handle_timeout(self, dispatcher):
+        dispatcher.connection.close()
+        self.remove_subscriptions()
+
     def _handle_destination(self, dispatcher, req_dest, request):
         """
         We could receive single message or batch of messages. We need
@@ -368,13 +374,15 @@ class StompClient(object):
     which tells client whether it should manage reactor's
     life cycle (by default set to True).
     """
-    def __init__(self, sock, reactor, owns_reactor=True):
+    def __init__(self, sock, reactor, owns_reactor=True,
+                 incoming_heartbeat=30000, outgoing_heartbeat=0):
         self._reactor = reactor
         self._owns_reactor = owns_reactor
         self._messageHandler = None
         self._socket = sock
 
-        self._aclient = stomp.AsyncClient()
+        self._aclient = stomp.AsyncClient(
+            incoming_heartbeat, outgoing_heartbeat)
         self._stompConn = _StompConnection(
             self,
             self._aclient,
@@ -582,7 +590,8 @@ def StompRpcClient(stomp_client, request_queue, response_queue):
     )
 
 
-def SimpleClient(host, port=54321, ssl=True):
+def SimpleClient(host, port=54321, ssl=True, incoming_heartbeat=30000,
+                 outgoing_heartbeat=0):
     """
     Returns JsonRpcClient able to receive jsonrpc messages and notifications.
     It is required to provide a host where we want to connect, port and whether
@@ -597,11 +606,13 @@ def SimpleClient(host, port=54321, ssl=True):
                                      ca_certs=constants.CA_FILE,
                                      protocol=sslutils.CLIENT_PROTOCOL)
     return StandAloneRpcClient(host, port, "jms.topic.vdsm_requests",
-                               str(uuid4()), sslctx, lazy_start=False)
+                               str(uuid4()), sslctx, False,
+                               incoming_heartbeat, outgoing_heartbeat)
 
 
 def StandAloneRpcClient(host, port, request_queue, response_queue,
-                        sslctx=None, lazy_start=True):
+                        sslctx=None, lazy_start=True, incoming_heartbeat=30000,
+                        outgoing_heartbeat=0):
     """
     Returns JsonRpcClient able to receive jsonrpc messages and notifications.
     It is required to provide host and port where we want to connect and
@@ -616,7 +627,8 @@ def StandAloneRpcClient(host, port, request_queue, response_queue,
         thread.start()
 
     client = StompClient(utils.create_connected_socket(host, port, sslctx),
-                         reactor)
+                         reactor, incoming_heartbeat=incoming_heartbeat,
+                         outgoing_heartbeat=outgoing_heartbeat)
 
     jsonclient = JsonRpcClient(
         ClientRpcTransportAdapter(
